# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

import re

from flask import Blueprint, render_template, flash, url_for, redirect, make_response
from flask.ext.login import login_required
from sqlalchemy.orm.exc import NoResultFound

from resultsdb_api import ResultsDBapi

from testdays import app, db
from testdays.controllers.main import preparse_results
import testdays.lib.wiki as wiki
from testdays.lib.forms import AddEventForm
from testdays.models.testday import Event, Category, Testcase, TESTCASE_TYPES

admin = Blueprint('admin', __name__)

RDB_API = None

@app.before_first_request
def before_first_request():
    global RDB_API
    RDB_API = ResultsDBapi(app.config['RESULTSDB_URL'])


@admin.route('/admin')
@admin.route('/admin/')
#@login_required
def admin_index():
    return render_template('admin/index.html')

def event_from_metadata(url):

    # grab the respective section from wikipage
    data = wiki.get_page_section(url, "TestdayApp Metadata")


    # matches text from lines enclosed in '=' characters
    re_groupname = re.compile("^=+\s*(.*?)\s*=*$")

    td = {"name":"", "url":"", "categories": []}
    category = {"name":"", "testcases": []}
    testcase = {"name":"", "url":"", "type":""}

    # splits text on newlines, strips whitespaces and filters relevant lines
    # the first line is deemed irrelevant, as it is just the name of the section
    data = [line.strip() for line in data.split('\n')[1:]]
    data = [line for line in data if line.startswith("*") or line.startswith("=")]

    # Testday Name & URL
    # the first line must contain testday's name and URL in the '*name;url' format
    name_url = data.pop(0)
    if not name_url.startswith("*"):
        return (False, "Missing Testday name & URL")
    try:
        # remove * at the beginning, split by ';' in two parts, and strip whitespaces
        td_name, td_url = [part.strip() for part in name_url[1:].split(';', 2)]
    except ValueError: # split failed
        return (False, "Testday name or URL missing")

    # sanity check
    if len(td_name) < 5:
        return (False, "Testday name must be longer than 5 characters")
    if not td_url.startswith("http"):
        return (False, "Testday URL must start with 'http'")

    if td_url.startswith("https"):
        td_url = td_url.replace("https", "http", 1)

    event = Event(td_name, url, td_url)

    # Testcase Sections
    category = None
    for line in data:
        # Create a new category, when we encounter category metadata
        if line.startswith("="):
            name = re_groupname.findall(line)[0]
            if not name:
                return (False, "Category name must be at least 1 character long at line %r" % line)
            category = Category(name, event)
            continue

        # skip until we get a category
        if category is None:
            continue
        # skip until we get a testcase
        if not line.startswith("*"):
            continue

        # Parse the testcase metadata

        # remove * at the beginning, split by ';' and strip whitespaces
        l = [part.strip() for part in line[1:].split(';')]
        # append default testcase result type (pass/fail/info)
        l.append('tc')

        if len(l) < 3:
            return (False, "Testcase name or url missing on line %r" % line)

        testcase = Testcase(l[0], l[1], l[2].lower(), category)

        if len(testcase.name) < 1:
            return (False, "Testcase name must be at least 1 character long at line %r" % line)
        if not testcase.url.startswith('http'):
            return (False, "Testcase url does not start with 'http' on line %r" % line)

        # convert unknown types to the default "testcase" type
        if testcase.type not in TESTCASE_TYPES:
            testcase.type = 'tc'

        # change https to http
        testcase.url = testcase.url.replace("https", "http", 1)

    return(True, event)


@admin.route('/admin/add_event', methods=['GET', 'POST'])
def add_event():
    form_ = AddEventForm()

    if form_.validate_on_submit():
        was_ok, data = event_from_metadata(form_.metadata_url.data)
        if was_ok:
            event = data
            try:
                old_event = db.session.query(Event).filter(Event.metadata_url == event.metadata_url).one()
            # Event with this metadata url not found -> create new event
            except NoResultFound:
                db.session.add(event)
                db.session.commit()
                db.session.add(event)

                # start resultsdb job
                rdb_job = RDB_API.create_job(
                        url_for('main.show_event', event_id=event.id),
                        name="Testday - %s" % event.testday_url.split('/')[-1],
                        )
                RDB_API.update_job(rdb_job['id'], status='RUNNING')

                event.resultsdb_job_id = rdb_job['id']
                db.session.add(event)
                db.session.commit()
                flash('Event was successfully created')
            # Event with this metadata url found -> update
            else:
                # Remove old categories and testcases
                for category in old_event.categories:
                    for testcase in category.testcases:
                        db.session.delete(testcase)
                    db.session.delete(category)

                event.id = old_event.id
                event.resultsdb_job_id = old_event.resultsdb_job_id
                db.session.merge(event)
                db.session.commit()
                flash('Event was successfully updated')

            return redirect(url_for('main.show_event', event_id = event.id))

        # an error was encountered, flash the error and show the add_event form
        flash(data, 'error')

    return render_template('admin/add_event.html',
                form = form_,
                form_action = url_for(".add_event"),
                header = u"Add/Edit Event",
            )

@admin.route('/admin/export_event', methods=['GET', 'POST'])
def export_event():
    form_ = AddEventForm()

    if form_.validate_on_submit():
        try:
            event = db.session.query(Event).filter(Event.metadata_url == form_.metadata_url.data).one()

        except NoResultFound:
                flash('Event not found', 'error')

        results = preparse_results(event)
        output = render_template('admin/export_event.txt', event=event, results=results)
        response = make_response(output)
        response.headers["Content-Type"] = "text/plain; charset=utf-8"
        return response


    return render_template('admin/add_event.html',
                form = form_,
                form_action = url_for(".export_event"),
                header = u"Export Event",
            )

