# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

import re
import json
from flask import Blueprint, render_template, redirect, url_for, flash, session

import resultsdb_api

from testdays import app, db
from testdays.models.testday import Event, Category, Testcase
from testdays.lib.forms import EnterResultForm

main = Blueprint('main', __name__)

RDB_API = None
TESTCASES = []
RE_TESTCASE_NAME_NORMALIZE = re.compile(r'[^a-zA-Z0-9_]')

@app.before_first_request
def before_first_request():
    global RDB_API
    RDB_API = resultsdb_api.ResultsDBapi(app.config['RESULTSDB_URL'])

@main.route('/')
@main.route('/index')
def index():
    return redirect(url_for('.events'))

@main.route('/events')
def events():
    events = db.session.query(Event).order_by(Event.created_at)
    return render_template('list_events.html', events=events)


# TODO: use Memcached instead of SimpleCache
from werkzeug.contrib.cache import SimpleCache
CACHE = SimpleCache()
CACHE_TIMEOUT = 10

def preparse_results(event):
    global CACHE

    cached = CACHE.get("event_%s_matrix" % event.id)
    if cached is not None:
        return cached

    results = RDB_API.get_job(event.resultsdb_job_id)['results']

    # results is a list of dicts like this:
    # {
    #   "href": "http://taskotron-local/resultsdb_api/api/v1.0/results/1510407",
    #   "id": 1510407,
    #   "job_url": "http://taskotron-local/resultsdb_api/api/v1.0/jobs/115516",
    #   "log_url": null,
    #   "outcome": "PASSED",
    #   "result_data": {
    #     "bugs": [
    #       ""
    #     ],
    #     "col_1": [
    #       "Fedora 19"
    #     ],
    #     "result": [
    #       "cats!"
    #     ],
    #     "username": [
    #       "jskladan"
    #     ]
    #   },
    #   "submit_time": "2015-05-14T09:20:36.392351",
    #   "summary": "Comment number 4",
    #   "testcase": {
    #     "href": "http://taskotron-local/resultsdb_api/api/v1.0/testcases/testdays_QA_Testcase_Power_Management_tuned_off_idle",
    #     "name": "testdays_QA_Testcase_Power_Management_tuned_off_idle",
    #     "url": "http://fedoraproject.org/wiki/QA:Testcase_Power_Management_tuned_off_idle"
    #   }
    # }

    # The following code will rearrange the results into structure like this one:
    #
    # {$username: {$col_1: {$tcurl: [{'bugs': $bugs.split(';'),
    #                                 'comment': $summary,
    #                                 'result': $result,
    #                                }]}}}
    #
    # Where $username, $col_1, and $result are from ['result_data'],
    #   $tcurl is the testcase url from ['testcase']['url']

    by_user = {}
    for result in results:
        data = result['result_data']

        username = data['username'][0]
        col_1 = data['col_1'][0]
        tcurl = result['testcase']['url']

        outcome = data['result'][0]
        bugs = []
        if data['bugs'][0]:
            bugs = data['bugs'][0].split(';')
        comment = result['summary']

        by_user.setdefault(username, {})
        by_user[username].setdefault(col_1, {})
        by_user[username][col_1].setdefault(tcurl, [])

        by_user[username][col_1][tcurl].append(
                dict(
                    result = outcome,
                    bugs = bugs,
                    comment = comment
                    )
                )

    # At this point, by_user contains something like this:
    # {'usr1': {'moofoo': {'http://test2': [{'bugs': [],
    #                                        'comment': '',
    #                                        'result': 'INFO',
    #                                       }]}},
    #  'usr2': {(foobar': {'http://test1': [{'bugs': [],
    #                                        'comment': None,
    #                                        'result': 'FAILED',
    #                                        }],
    #                      'http://test2': [{'bugs': ['12345',
    #                                                 '6431'],
    #                                        'comment': 'Does not work!',
    #                                        'result': 'PASSED',
    #                                        }]}}}

    # We'll  prepare the data even further - the event's
    #  testcases are split into categories. We'll prepare the by-category
    #  matrix, se the template to show the results can stay fairly simple
    # The result of the transformation is:
    #  {1: [{
    #        'username': u'jskladan',
    #        'profile': u'Ubuntu 4.1',
    #        'results': [[(u'FAILED', 1), (u'PASSED', 2)], []],
    #        'comments': [([u'123', u'456'], u'Comment number 2'),
    #                     ([], u'Comment number 1')],
    #       },
    #       {
    #        'username': u'jskladan',
    #        'profile': u'Fedora 19',
    #        'results': [[(u'INFO', False), (u'PASSED', 1)], [(u'cats!', 2)]],
    #        'comments': [([], u'Comment number 3'), ([], u'Comment number 4')],
    #       }],
    #   2: []}
    #
    # $results represents the results per testcase - each sublist of the top list
    #   'maps' (positionaly) to the Testcase in the respective Category. It can
    #   contain multiple tuples, each representing one result (one user with one
    #   profile can submit multiple results per Testcase). The first item in the
    #   tuple is the actual result. The second item is either False or Int.
    #     False means that the result has no bugs/comment associated with it.
    #     Int is the index+1 of the comment in the $comments field. This way
    #       it is very simple to create link-references from result to comment/bugs
    #
    # $comments is a list of tuples, each tuple represents bugs/comment for the
    #  respective result (see above).
    #  First item in the tuple is a list of BUG numbers
    #  Second item is the comment itself.

    matrix = {}
    # We want to have the users sorted alphabetically in the table
    users = by_user.keys()
    users.sort()

    # Create per-category pre-parsed lines of results
    for category in event.categories:
        matrix[category.id] = []
        relevant_testcases = set([tc.url for tc in category.testcases])

        # Create a line-of-results per each user's profile
        for user in users:
            # Also, sort by the profile-info alphabetically
            user_profiles = by_user[user].keys()
            user_profiles.sort()
            for profile in user_profiles:
                data = by_user[user][profile]
                # Check whether this user-profile set any results
                #  for testcases from this category
                if not relevant_testcases.intersection(data.keys()):
                    continue

                # Prepare the per-line data
                #  see the explanation of the structure above
                line = dict(
                        username=user,
                        profile=profile,
                        results=[],
                        comments=[])

                comment_id = 0
                for testcase in category.testcases:
                    # Append new results-container for the respective
                    #  testcase.
                    results_by_tc = []
                    line['results'].append(results_by_tc)

                    # If the user-profile did not fill any result for this
                    #   specific testcase, go to the next one.
                    # The "no results filled" state is represented by the empty
                    #   list.
                    if testcase.url not in data:
                        continue

                    # Go through all the details of the results for this
                    #   user-profile-testcase combination
                    for detail in data[testcase.url]:
                        has_comment = bool(detail['bugs'] or detail['comment'])
                        # If there are bugs and/or comments for the result,
                        #   add them to the line container
                        if has_comment:
                            comment_id += 1
                            line['comments'].append((detail['bugs'], detail['comment']))
                            has_comment = comment_id
                        results_by_tc.append((detail['result'], has_comment))
                matrix[category.id].append(line)

    CACHE.set("event_%s_matrix" % event.id, matrix, CACHE_TIMEOUT)
    return matrix


@main.route('/events/<int:event_id>')
def show_event(event_id):
    event = db.session.query(Event).filter(Event.id == event_id).one()
    results = preparse_results(event)

    prof = json.loads(session.get('profile', '{}'))
    username = prof.get('username')
    col_1 = prof.get('col_1')
    filled = prof.get('filled', {}).get(username, {}).get(col_1, [])
    profile = dict(username=username, col_1=col_1, filled=filled)

    return render_template('show_event.html', event=event, results=results,
            profile=profile)

def testcase_to_name(testcase):
    prefix = "testdays"
    name = testcase.url.split('/')[-1]
    name = RE_TESTCASE_NAME_NORMALIZE.sub('_', name)
    return "%s_%s" % (prefix, name)

def ensure_testcase(testcase):
    global TESTCASES

    name = testcase_to_name(testcase)
    if name not in TESTCASES:
        try:
            RDB_API.get_testcase(name)
        except resultsdb_api.ResultsDBapiException:
            try:
                RDB_API.create_testcase(name, testcase.url)
            except resultsdb_api.ResultsDBapiException:
                raise
        TESTCASES.append(name)
    return name


def update_profile_cookies(username, col_1, profile=None):
    if not profile:
        profile = json.loads(session.get('profile', '{}'))
    profile['username'] = username
    profile['col_1'] = col_1
    filled = profile.setdefault('filled', {})
    filled.setdefault(username, {})
    filled[username].setdefault(col_1, [])

    session['profile'] = json.dumps(profile)
    return profile

def update_filled_cookies(username, col_1, testcase_name, profile=None):
    profile = update_profile_cookies(username, col_1, profile)
    profile['filled'][username][col_1].append(testcase_name)

    session['profile'] = json.dumps(profile)
    return profile


@main.route('/events/<int:event_id>/enter_result/<int:tc_id>', methods=['GET', 'POST'])
def enter_result(event_id, tc_id):
    testcase = db.session.query(Testcase).filter(Testcase.id == tc_id).one()
    form_ = EnterResultForm(testcase)

    session_profile = json.loads(session.get('profile', '{}'))

    if form_.validate_on_submit():
        outcome = form_.result.data
        if testcase.type == 'txt':
            outcome = 'PASSED'

        testcase_name = ensure_testcase(testcase)

        result_dict = dict(
                job_id=testcase.category.event.resultsdb_job_id,
                testcase_name=testcase_name,
                outcome=outcome,
                result=form_.result.data,
                col_1=form_.col_1.data,
                username=form_.username.data,
                bugs=form_.bugs.data,
                summary=form_.comment.data,
                )
        try:
            r = RDB_API.create_result(**result_dict)
            flash("Result Saved")
            update_filled_cookies(form_.username.data, form_.col_1.data, testcase.name,  session_profile)
            return redirect(url_for('.show_event', event_id=event_id))
        except resultsdb_api.ResultsDBapiException as e:
            update_profile_cookies(form_.username.data, form_.col_1.data, session_profile)
            flash("There was an error when saving the result (%s)" % e.message, 'error')

    if not form_.errors:
        if not form_.username.data:
            form_.username.data = session_profile.get('username', '')
        if not form_.col_1.data:
            form_.col_1.data = session_profile.get('col_1', '')

    return render_template('enter_result.html',
            form = form_,
            form_action = url_for('.enter_result', event_id=event_id, tc_id=tc_id),
            header = "Enter result",
            )













